/*@ gen-tcaps.h, generated by make-tcap-map.pl.
 *@ See termcap.c for more */

static char const a_termcap_namedat[] = {
   /* [0]+8, mx_TERMCAP_CMD_te */ 't','e',  'r','m','c','u','p','\0',
   /* [8]+8, mx_TERMCAP_CMD_ti */ 't','i',  's','m','c','u','p','\0',
   /* [16]+7, mx_TERMCAP_CMD_ks */ 'k','s',  's','m','k','x','\0',
   /* [23]+7, mx_TERMCAP_CMD_ke */ 'k','e',  'r','m','k','x','\0',
   /* [30]+5, mx_TERMCAP_CMD_ce */ 'c','e',  'e','l','\0',
   /* [35]+6, mx_TERMCAP_CMD_ch */ 'c','h',  'h','p','a','\0',
   /* [41]+5, mx_TERMCAP_CMD_cr */ 'c','r',  'c','r','\0',
   /* [46]+7, mx_TERMCAP_CMD_le */ 'l','e',  'c','u','b','1','\0',
   /* [53]+7, mx_TERMCAP_CMD_nd */ 'n','d',  'c','u','f','1','\0',
   /* [60]+5, mx_TERMCAP_CMD_cd */ 'c','d',  'e','d','\0',
   /* [65]+7, mx_TERMCAP_CMD_ho */ 'h','o',  'h','o','m','e','\0',
   /* [72]+8, mx_TERMCAP_CMD_cl */ 'c','l',  'c','l','e','a','r','\0',
   /* [80]+5, mx_TERMCAP_QUERY_am */ 'a','m',  'a','m','\0',
   /* [85]+6, mx_TERMCAP_QUERY_sam */ 'Y','E',  's','a','m','\0',
   /* [91]+7, mx_TERMCAP_QUERY_xenl */ 'x','n',  'x','e','n','l','\0',
   /* [98]+9, mx_TERMCAP_QUERY_colors */ 'C','o',  'c','o','l','o','r','s','\0',
#ifdef mx_HAVE_KEY_BINDINGS
   /* [107]+6, mx_TERMCAP_QUERY_key_backspace */ 'k','b',  'k','b','s','\0',
   /* [113]+8, mx_TERMCAP_QUERY_key_dc */ 'k','D',  'k','d','c','h','1','\0',
   /* [121]+6, mx_TERMCAP_QUERY_key_sdc */ '*','4',  'k','D','C','\0',
   /* [127]+6, mx_TERMCAP_QUERY_key_eol */ 'k','E',  'k','e','l','\0',
   /* [133]+7, mx_TERMCAP_QUERY_key_exit */ '@','9',  'k','e','x','t','\0',
   /* [140]+8, mx_TERMCAP_QUERY_key_ic */ 'k','I',  'k','i','c','h','1','\0',
   /* [148]+6, mx_TERMCAP_QUERY_key_sic */ '#','3',  'k','I','C','\0',
   /* [154]+8, mx_TERMCAP_QUERY_key_home */ 'k','h',  'k','h','o','m','e','\0',
   /* [162]+7, mx_TERMCAP_QUERY_key_shome */ '#','2',  'k','H','O','M','\0',
   /* [169]+7, mx_TERMCAP_QUERY_key_end */ '@','7',  'k','e','n','d','\0',
   /* [176]+7, mx_TERMCAP_QUERY_key_send */ '*','7',  'k','E','N','D','\0',
   /* [183]+6, mx_TERMCAP_QUERY_key_npage */ 'k','N',  'k','n','p','\0',
   /* [189]+6, mx_TERMCAP_QUERY_key_ppage */ 'k','P',  'k','p','p','\0',
   /* [195]+8, mx_TERMCAP_QUERY_key_left */ 'k','l',  'k','c','u','b','1','\0',
   /* [203]+7, mx_TERMCAP_QUERY_key_sleft */ '#','4',  'k','L','F','T','\0',
   /* [210]+8, mx_TERMCAP_QUERY_xkey_aleft */ '\0','\0',  'k','L','F','T','3','\0',
   /* [218]+8, mx_TERMCAP_QUERY_xkey_cleft */ '\0','\0',  'k','L','F','T','5','\0',
   /* [226]+8, mx_TERMCAP_QUERY_key_right */ 'k','r',  'k','c','u','f','1','\0',
   /* [234]+7, mx_TERMCAP_QUERY_key_sright */ '%','i',  'k','R','I','T','\0',
   /* [241]+8, mx_TERMCAP_QUERY_xkey_aright */ '\0','\0',  'k','R','I','T','3','\0',
   /* [249]+8, mx_TERMCAP_QUERY_xkey_cright */ '\0','\0',  'k','R','I','T','5','\0',
   /* [257]+8, mx_TERMCAP_QUERY_key_down */ 'k','d',  'k','c','u','d','1','\0',
   /* [265]+6, mx_TERMCAP_QUERY_xkey_sdown */ '\0','\0',  'k','D','N','\0',
   /* [271]+7, mx_TERMCAP_QUERY_xkey_adown */ '\0','\0',  'k','D','N','3','\0',
   /* [278]+7, mx_TERMCAP_QUERY_xkey_cdown */ '\0','\0',  'k','D','N','5','\0',
   /* [285]+8, mx_TERMCAP_QUERY_key_up */ 'k','u',  'k','c','u','u','1','\0',
   /* [293]+6, mx_TERMCAP_QUERY_xkey_sup */ '\0','\0',  'k','U','P','\0',
   /* [299]+7, mx_TERMCAP_QUERY_xkey_aup */ '\0','\0',  'k','U','P','3','\0',
   /* [306]+7, mx_TERMCAP_QUERY_xkey_cup */ '\0','\0',  'k','U','P','5','\0',
   /* [313]+6, mx_TERMCAP_QUERY_kf0 */ 'k','0',  'k','f','0','\0',
   /* [319]+6, mx_TERMCAP_QUERY_kf1 */ 'k','1',  'k','f','1','\0',
   /* [325]+6, mx_TERMCAP_QUERY_kf2 */ 'k','2',  'k','f','2','\0',
   /* [331]+6, mx_TERMCAP_QUERY_kf3 */ 'k','3',  'k','f','3','\0',
   /* [337]+6, mx_TERMCAP_QUERY_kf4 */ 'k','4',  'k','f','4','\0',
   /* [343]+6, mx_TERMCAP_QUERY_kf5 */ 'k','5',  'k','f','5','\0',
   /* [349]+6, mx_TERMCAP_QUERY_kf6 */ 'k','6',  'k','f','6','\0',
   /* [355]+6, mx_TERMCAP_QUERY_kf7 */ 'k','7',  'k','f','7','\0',
   /* [361]+6, mx_TERMCAP_QUERY_kf8 */ 'k','8',  'k','f','8','\0',
   /* [367]+6, mx_TERMCAP_QUERY_kf9 */ 'k','9',  'k','f','9','\0',
   /* [373]+7, mx_TERMCAP_QUERY_kf10 */ 'k',';',  'k','f','1','0','\0',
   /* [380]+7, mx_TERMCAP_QUERY_kf11 */ 'F','1',  'k','f','1','1','\0',
   /* [387]+7, mx_TERMCAP_QUERY_kf12 */ 'F','2',  'k','f','1','2','\0',
   /* [394]+7, mx_TERMCAP_QUERY_kf13 */ 'F','3',  'k','f','1','3','\0',
   /* [401]+7, mx_TERMCAP_QUERY_kf14 */ 'F','4',  'k','f','1','4','\0',
   /* [408]+7, mx_TERMCAP_QUERY_kf15 */ 'F','5',  'k','f','1','5','\0',
   /* [415]+7, mx_TERMCAP_QUERY_kf16 */ 'F','6',  'k','f','1','6','\0',
   /* [422]+7, mx_TERMCAP_QUERY_kf17 */ 'F','7',  'k','f','1','7','\0',
   /* [429]+7, mx_TERMCAP_QUERY_kf18 */ 'F','8',  'k','f','1','8','\0',
   /* [436]+7, mx_TERMCAP_QUERY_kf19 */ 'F','9',  'k','f','1','9','\0',
#endif /* mx_HAVE_KEY_BINDINGS */
};

static struct a_termcap_control const a_termcap_control[] = {
# ifdef mx_HAVE_TERMCAP
   {/* 0. mx_TERMCAP_CMD_te */ mx_TERMCAP_CAPTYPE_STRING, 0},
   {/* 1. mx_TERMCAP_CMD_ti */ mx_TERMCAP_CAPTYPE_STRING, 8},
   {/* 2. mx_TERMCAP_CMD_ks */ mx_TERMCAP_CAPTYPE_STRING, 16},
   {/* 3. mx_TERMCAP_CMD_ke */ mx_TERMCAP_CAPTYPE_STRING, 23},
# endif
# ifdef mx_HAVE_MLE
   {/* 4. mx_TERMCAP_CMD_ce */ mx_TERMCAP_CAPTYPE_STRING, 30},
   {/* 5. mx_TERMCAP_CMD_ch */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_ARG_IDX1, 35},
   {/* 6. mx_TERMCAP_CMD_cr */ mx_TERMCAP_CAPTYPE_STRING, 41},
   {/* 7. mx_TERMCAP_CMD_le */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_ARG_CNT, 46},
   {/* 8. mx_TERMCAP_CMD_nd */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_ARG_CNT, 53},
#  ifdef mx_HAVE_TERMCAP
   {/* 9. mx_TERMCAP_CMD_cd */ mx_TERMCAP_CAPTYPE_STRING, 60},
   {/* 10. mx_TERMCAP_CMD_ho */ mx_TERMCAP_CAPTYPE_STRING, 65},
#  endif
   {/* 11. mx_TERMCAP_CMD_cl */ mx_TERMCAP_CAPTYPE_STRING, 72},
# endif
   {/* 12. mx_TERMCAP_QUERY_am */ mx_TERMCAP_CAPTYPE_BOOL|a_TERMCAP_F_QUERY, 80},
   {/* 13. mx_TERMCAP_QUERY_sam */ mx_TERMCAP_CAPTYPE_BOOL|a_TERMCAP_F_QUERY, 85},
   {/* 14. mx_TERMCAP_QUERY_xenl */ mx_TERMCAP_CAPTYPE_BOOL|a_TERMCAP_F_QUERY, 91},
# ifdef mx_HAVE_COLOUR
   {/* 15. mx_TERMCAP_QUERY_colors */ mx_TERMCAP_CAPTYPE_NUMERIC|a_TERMCAP_F_QUERY, 98},
# endif
# ifdef mx_HAVE_KEY_BINDINGS
   {/* 16. mx_TERMCAP_QUERY_key_backspace */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 107},
   {/* 17. mx_TERMCAP_QUERY_key_dc */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 113},
   {/* 18. mx_TERMCAP_QUERY_key_sdc */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 121},
   {/* 19. mx_TERMCAP_QUERY_key_eol */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 127},
   {/* 20. mx_TERMCAP_QUERY_key_exit */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 133},
   {/* 21. mx_TERMCAP_QUERY_key_ic */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 140},
   {/* 22. mx_TERMCAP_QUERY_key_sic */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 148},
   {/* 23. mx_TERMCAP_QUERY_key_home */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 154},
   {/* 24. mx_TERMCAP_QUERY_key_shome */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 162},
   {/* 25. mx_TERMCAP_QUERY_key_end */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 169},
   {/* 26. mx_TERMCAP_QUERY_key_send */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 176},
   {/* 27. mx_TERMCAP_QUERY_key_npage */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 183},
   {/* 28. mx_TERMCAP_QUERY_key_ppage */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 189},
   {/* 29. mx_TERMCAP_QUERY_key_left */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 195},
   {/* 30. mx_TERMCAP_QUERY_key_sleft */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 203},
   {/* 31. mx_TERMCAP_QUERY_xkey_aleft */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 210},
   {/* 32. mx_TERMCAP_QUERY_xkey_cleft */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 218},
   {/* 33. mx_TERMCAP_QUERY_key_right */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 226},
   {/* 34. mx_TERMCAP_QUERY_key_sright */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 234},
   {/* 35. mx_TERMCAP_QUERY_xkey_aright */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 241},
   {/* 36. mx_TERMCAP_QUERY_xkey_cright */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 249},
   {/* 37. mx_TERMCAP_QUERY_key_down */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 257},
   {/* 38. mx_TERMCAP_QUERY_xkey_sdown */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 265},
   {/* 39. mx_TERMCAP_QUERY_xkey_adown */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 271},
   {/* 40. mx_TERMCAP_QUERY_xkey_cdown */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 278},
   {/* 41. mx_TERMCAP_QUERY_key_up */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 285},
   {/* 42. mx_TERMCAP_QUERY_xkey_sup */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 293},
   {/* 43. mx_TERMCAP_QUERY_xkey_aup */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 299},
   {/* 44. mx_TERMCAP_QUERY_xkey_cup */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 306},
   {/* 45. mx_TERMCAP_QUERY_kf0 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 313},
   {/* 46. mx_TERMCAP_QUERY_kf1 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 319},
   {/* 47. mx_TERMCAP_QUERY_kf2 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 325},
   {/* 48. mx_TERMCAP_QUERY_kf3 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 331},
   {/* 49. mx_TERMCAP_QUERY_kf4 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 337},
   {/* 50. mx_TERMCAP_QUERY_kf5 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 343},
   {/* 51. mx_TERMCAP_QUERY_kf6 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 349},
   {/* 52. mx_TERMCAP_QUERY_kf7 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 355},
   {/* 53. mx_TERMCAP_QUERY_kf8 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 361},
   {/* 54. mx_TERMCAP_QUERY_kf9 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 367},
   {/* 55. mx_TERMCAP_QUERY_kf10 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 373},
   {/* 56. mx_TERMCAP_QUERY_kf11 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 380},
   {/* 57. mx_TERMCAP_QUERY_kf12 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 387},
   {/* 58. mx_TERMCAP_QUERY_kf13 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 394},
   {/* 59. mx_TERMCAP_QUERY_kf14 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 401},
   {/* 60. mx_TERMCAP_QUERY_kf15 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 408},
   {/* 61. mx_TERMCAP_QUERY_kf16 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 415},
   {/* 62. mx_TERMCAP_QUERY_kf17 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 422},
   {/* 63. mx_TERMCAP_QUERY_kf18 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 429},
   {/* 64. mx_TERMCAP_QUERY_kf19 */ mx_TERMCAP_CAPTYPE_STRING|a_TERMCAP_F_QUERY, 436},
# endif /* mx_HAVE_KEY_BINDINGS */
};
